import { createElement, useContext, useState, useRef, useMemo, createContext } from 'react';

function createHoc(hook) {
    return (map) => (component) => {
        const hoc = (props) => {
            const [state, actions] = hook();
            return createElement(component, Object.assign({}, props, map(state, actions)));
        };
        hoc.displayName = `WithDakpan(${component.displayName || component.name || 'Component'})`;
        return hoc;
    };
}

function createHook(context) {
    return (nullable) => {
        const state = useContext(context);
        if (!state && !nullable) {
            throw Error('Provider not mounted');
        }
        return [
            state && state.state,
            state && state.actions
        ];
    };
}

function createProvider(context, actions, state) {
    return ({ children, value }) => {
        const [, update] = useState({});
        const stateRef = useRef(state || value);
        const mapped = useMemo(() => Object.keys(actions).reduce((previous, current) => {
            const execute = (...args) => () => Promise.resolve(actions[current](...args)(stateRef.current))
                .then((next) => {
                if (next) {
                    stateRef.current = next;
                    update({});
                }
            });
            const action = (...args) => execute(...args)();
            action.c = execute;
            return Object.assign({}, previous, { [current]: action });
        }, {}), [actions, stateRef]);
        return createElement(context.Provider, {
            value: {
                state: stateRef.current,
                actions: mapped
            }
        }, children);
    };
}

function createDakpan(initialState) {
    return (actions) => {
        const context = createContext(undefined);
        const hook = createHook(context);
        const state = initialState instanceof Function ? initialState() : initialState;
        return [
            createProvider(context, actions, state),
            hook,
            createHoc(hook)
        ];
    };
}

export { createDakpan };
//# sourceMappingURL=index.es.js.map
